/* measure conductance of water solution, and temperature
  first beta at 13 feb 2018
  by Dr. Giovanni Visco
  Rome University
  // ~!@#$%^&*()_+{}:"|?><|`-=[]\';/.,“”
  // !"#$%&'()*+,-./:;<=>?@[\]^_`{|}~ ¡¢£¤¥¦§¨©ª«¬­®¯°±²³´µ¶·¸¹º»¼½¾¿×÷
  licence!, see https://creativecommons.org/licenses/by-nc-nd/4.0/

  ** the conductmeter **
  2 stainless steel electrode (spiral ~13mm) on the 2 sides of a PCB bar
  supply with square wave, ~1KHz, 5V pp to the ground
  in the circuit, published, one of the electrode is put at 2.5V to obtain + / - on the other
  for schematic see: http://www.cma4ch.org/chemo/lessons/eserc13_2015-16.html
    in this test D9= , R1v=10Kohm , R2v=1Mohm, C1=15nF, R1=220ohm, C2=0.15uF
    if you see components side the pinout is, front view, from left (must go to Arduino)
      on the PCB: +5V     ! gnd     ! out    ! square wave input
      on Arduino: pin +5V ! pin gnd ! pin A0 ! pin CH5
    if you see components side the pinout is, front view, from left (to Electrodes)
      on the PCB: B electrode ! shield for B ! A electrode

  ** the conduct measure **
  build a new (jun18) function to measure resistance between electrodes as:
  oversampling to 12-bits by oversampling-and-decimation with sum of 16 reading and division by 4
  after average of 9 measure to produce a single SensorValue
  the value of R1 current limiter, R2 load of rectifier, C1-R3-C2 smoothing, all can be changed in function of water conductivity
  the following table can be used as starting values
  water from 0 to 500 uS = R1=10Kohm , R2=1Mohm and C1=2.2nF
  water from 200 to 1500 uS = R1=1Kohm , R2=1Mohm and C1=15nF
  water from 900 to 2500 uS = R1=560ohm , R2=2Mohm and C1=22nF
  Is better print the results to the serial monitor or serial plotter (or use a software to capture the serial data)
  
  ** measure temperature **
  MCP970x is a Tiny Analog Temperature Sensor on TO-92, SC70-5, SOT-23-5
  with Range: -40C to +125C or -40C to +150C (High Temperature MCP9700/9700A)
  with Accuracy: +/-2C at 0C to +70C (MCP9700A/9701A) and +/-4C (max.) for MCP9700/9701
  Optimized for Analog-to-Digital Converters (ADCs) as
  10.0 mV/C (typical) MCP9700/9700A and 19.5 mV/C (typical) MCP9701/9701A
  pinout bottom view
   /1| +5V ------------------------#--------#-----
  ( 2| out ---[2K2]-------->       C=10uF   0u1
   \3| gnd ------------------------#--------#-----
     
  The connection to Arduino UNO R3:
  pin1 to Arduino 5V --------------------#----
  pin2 to Arduino A2/CH16, input         C=10uF (on the output connector)
  pin3 to Arduino gnd -------------------#----
  
  ** the licence **
  This example code is in the public domain but with COPYRIGHT
  YOU CAN NOT USE IN ANY FORM WITHOUT CITATION OF AUTHOR

* This program is free software; you can redistribute it and/or modify    *
* it under the terms of the GNU General Public License as published by    *
* the Free Software Foundation; either version 3 of the following License *
* Creative Commons Public License, CC-BY-NC-ND 4.0 EN                     *
******************************************************************* 
* This program is distributed in the hope that it will be useful, *
* but WITHOUT ANY WARRANTY; without even the implied warranty of  * 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            *                          *
* See the GNU General Public License for more details.            *
******************************************************************* 

  rewrite from scratch at 08 September 2017
  revision 13 feb 2018, revision 27 jun 2018, revision 15 oct 2018
  revision 10 jul 2019, revision 30 oct 2020, revision 09 apr 2021
  last revision 27 mar 2023
  by G. Visco, Rome University, La Sapienza
*/
// the declarations stay before void setup
// * pinout setting *
#define TempPin 16                  // the pin connected to signal from MCP9701 (ch16=A2)
#define MvPin 14                    // the pin to read mV from electrodes (ch14=A0)
#define HzPin 5                     // the square wave out from this pin
#define between 2775                // delay between measure, as example 3000-(70+155)ms 
#define Duty 127                    // 50% to generate a square wave
#define V1_1 1                      // 1=use 1.1V as reference, 0=use 5V as analogReference (for MCP970x)

// *** change and do an upload ***
char StringSol[] = "at Rome, Tap Water";   // what "solution" we are cheking ?
char compile_date[] = __DATE__;           // date of compile
char compile_time[] = __TIME__;           // time of compile

// *** function/variables setting ***
// ReadTemp;             // function: read mV from MCP9701 (average and oversampling by2 the value of measure pin)
  // ReadTemp have parameters and variables
  int TempBit = 1;       // the value (volts by ADC) of temperature from 0 to 2048 (return to caller)
  byte AverA = 0;        // used in for-loop in ReadTemp for average
  byte AverB = 0;        // used in for-loop in ReadTemp for oversampling
  float SummA = 1;       // used in the for-loop in ReadTemp to summa the values
  float SummB = 1;       // used in the for-loop in ReadTemp to summa the values
// PrnParam();           // function: print on serial the setting parameter
  // PrnParam not return values
// PrnHeader();          // function: print on serial the Header
  // Prneader not return values
// ReadResist();         // function: read mV from electrodes, a.u.
  // ReadResist have parameters and variables
  int SensorValue = 1;   // the input analog value from 0 to 2048
  // AverA, AverB, SummA, SummB already set in previous
// more variables setting
  unsigned int MeasNum =0; // count the number of measure
  int Count = 0;         // the counter for-loop
  float SummV = 0;       // used in for-loop to summ the values
  float SummT = 0;       // used in for-loop to summ the values

// the setup routine runs once when you press reset:
void setup() {
  analogReference(DEFAULT);        // remember to fit the capacitor to Gnd
  Serial.begin(19200, SERIAL_8N1); // open serial communication, 19200 baud
  pinMode(LED_BUILTIN,OUTPUT);     // the internal LED
  digitalWrite(LED_BUILTIN, LOW);  // turn the 13 LOW
  analogWrite(HzPin, Duty);        // analog write on pin 5 (the square wave, see Duty)
  SensorValue = analogRead(TempPin); // only to set and open the port (value from temperature)
  delay(5);                        // delay in ms for stability
  SensorValue = analogRead(MvPin); // only to set and open the port (value from electrode)
  PrnParam();                      // print the parameters, one time only
  delay(500);                      // delay in ms 
  PrnHeader();                     // print the header, one time only
  MeasNum =1;                      // count the number of measure
}
  
// the loop routine runs over and over again forever:
void loop() 
{
SummV = 0;                      // 0 for sum the volt/electrodes
SummT = 0;                      // 0 for sum temperature
for (Count = 0; Count < 3; Count++)
{
ReadResist();                   // read SensorValue, electrodes
SummV = SensorValue + SummV;    // sum the volt/electrodes
ReadTemp();                     // read TempBit, temperature
SummT = TempBit + SummT;        // sum the temperature
//Serial.print("\t");           // tab
Serial.print(MeasNum);          // print out the value of counter
Serial.print("\t");             // tab
Serial.print(SensorValue);      // print out the value of conductivity
Serial.print("\t\t");           // tab
Serial.println(TempBit);        // print out the temperature value
delay(between);                 // delay in ms for stability
MeasNum++;                      // number +1
}
SummV = SummV / 3;              // average the volt/electrodes
SummT = SummT / 3;              // average the temperature
Serial.print(MeasNum - 1);      // print out the value of counter
Serial.print("\t");             // tab
Serial.print(SummV);            // print average conductivity
Serial.print("\t\t");           // tab
Serial.print(SummT);            // print average temperature
Serial.print("\t");             // tab
Serial.println("avg");          // this line is an average
}

// all the new functions-declaration here
void PrnHeader()
{
// Purpose: Print the measure header in serial plot
// Parameters: none
// Return value: none
// Description: 3 line of the header to describe the data in columns
// Serial.print("\a");          // bell sound
Serial.println("conductivity and temperature measure, (c) GV55, v.R1.2, 2023"); // print header
Serial.print("measure");        // measure n.
Serial.print("\t");             // tab
Serial.print("conductivity");   // from electrodes a.u.
Serial.print("\t");             // tab
Serial.println("temperature");  // temperature, a.u.
Serial.print("(num)");          // 2nd line of header
Serial.print("\t");
Serial.print("(a.u.)");
Serial.print("\t\t");
Serial.println("(a.u.)");
return;
}

void PrnParam()
{
// Purpose: Print out the setting parameters
// in parameters: more and more
// Return value: none
// Description: many lines with tab
// Serial.print("\f");       // form feed
Serial.println("##################################################");
Serial.println("# measure conductivity/temperature of a solution #");
Serial.println("#\t\t\t\t\t\t #");
Serial.println("# Author:\tDr. Giovanni Visco\t\t #");
Serial.println("# At:\t\tLa Sapienza Rome University\t #");
Serial.println("# Version:\tver.R1.2 oversample/avg, mar2023 #");
Serial.println("# Copyright Creative Commons Public\t\t #");
Serial.println("# License under, CC BY-NC-ND 4.0 EN\t\t #");
Serial.print("# Run with solution:");
Serial.print(" ");
Serial.print(StringSol);
Serial.println("\t#");
Serial.print("# Compiled at:");
Serial.print("\t");
Serial.print(compile_date);
Serial.print("\t");
Serial.print(compile_time);
Serial.println("\t #");
Serial.print("# Wave out from Arduino at:");
Serial.print("\t\t");
Serial.print(HzPin);
Serial.println("\t #");
Serial.print("# value from electrode at:");
Serial.print("\t\t");
Serial.print(MvPin);
Serial.println("\t #");
Serial.print("# value from temperature sens. at:");
Serial.print("\t");
Serial.print(TempPin);
Serial.println("\t #");
Serial.print("# delay between measure (s):");
Serial.print("\t\t");
Serial.print((between + 70 + 155) / 1000);
Serial.println("\t #");
Serial.println("##################################################");
Serial.println("");       // close with blank line
return;
}

int ReadResist()
// Purpose: Determine the value of resistance between electrodes, DC from D9
// In parameters: MvPin
// Return value: SensorValue = from 0 to 4096 in less to 155ms (3x((16x3)+3))
// Description: sum 16 values and /4 (oversampling/decimation) and, after, 3 fast reads and average, 
{
SummB = 0;                                     // reset the counter before new loop
  for (AverA = 0; AverA < 3; AverA++)          // for loop for average
  {
    SummA = 0;                                 // reset the counter (reused) before loop
      for (AverB = 0; AverB < 16; AverB++)     // for loop for decimation
      {
        analogRead(MvPin);                     // sets the MUX and starts the conversion immediately after
        delay(1);                              // a tips & tricks to use analogRead with high impedance output        
        SummA = (SummA + analogRead(MvPin));   // sum AverB values (4) to oversampling (if we have noise)
        delay(2);                              // a little delay to hook the noise
      }
    SummB = (SummB + (SummA / 4.0));           // div by 4 (decimation), careful INT do the truncate itself, better use round
    delay(3);                                  // better 5 or more but with oversampling we must do fast reads
  }
SummB = (SummB / 3.0);                         // div by 3 (average)
SummB = (SummB + 0.5);                         // add 0.5 to better round next Cast function
SensorValue = (int) (SummB);                   // with (cast) we can do a round and convert to integer
return SensorValue;
}

int ReadTemp()
{
// Purpose: read the voltage from MCP9701A
// In parameters: TempPin
// Return value: TempBit = from 0 to 2046 in less then 70ms (3+3+3x((4x4)+3)+5)
// Description: AREF was set on void setup, here read the analog pin
// Cautions: AREF is set for all analogpins so here we have the same Vreference of ReadMv()
// Cautions: If doing math with floats, you need to add a decimal point, otherwise it will be treated as an Int 
if (V1_1 == 1)
{
  analogReference(INTERNAL);                  // set the reference to 1.1V
  analogRead(TempPin);                        // sets the MUX and starts the conversion immediately after
  delay(3);                                   // a tips & tricks to use analogRead with high impedance output       
  analogRead(TempPin);                        // sets the MUX and starts the conversion immediately after
  delay(3);                                   // a tips & tricks to use analogRead with high impedance output
}
 TempBit = 0;                                  // set to 0 before the for-cycle
 SummB = 0;                                    // reset the counter before new loop
  for (AverA = 0; AverA < 3; AverA++)          // for loop for average, 3 time
  {
    SummA = 0;                                 // reset the counter before new loop
      for (AverB = 0; AverB < 4; AverB++)      // for loop for decimation, 4 pass is setting here
      {
        analogRead(TempPin);                   // sets the MUX and starts the conversion immediately after
        delay(1);                              // a tips & tricks to use analogRead with high impedance output       
        SummA = (SummA + analogRead(TempPin)); // sum 4 values to oversampling (if we have noise)
        delay(3);                              // a little delay to hook the noise, 4ms
      }
    SummB = (SummB + (SummA / 2.0));           // div by 2 (decimation), careful INT do the truncate itself, better use round
    delay(3);                                  // better 5 or more but with oversampling we must do fast reads
  }
SummB = (SummB / 3.0);                         // div by 3 (average)
SummB = (SummB + 0.5);                         // add 0.5 to better round next Cast function
TempBit = (int) (SummB);                       // with (cast) we can do a round and convert to integer
analogReference(DEFAULT);                      // set the reference to 5V
analogRead(TempPin);                           // sets the MUX and starts the conversion immediately after
delay(5);                                      // a tips & tricks to use analogRead with high impedance output
return TempBit;
}

// not used
void BisBlink()
// Purpose: blink 2 times (the sketch pass here, so it is running)
// Parameters: none
// Return value: none but delay of 1.0sec
// Description: a beautiful blink 2 times for 1.0s
{
digitalWrite(LED_BUILTIN, HIGH);        // turn the LED on
delay(150);                             // 150ms
digitalWrite(LED_BUILTIN, LOW);         // turn the LED off
delay(200);                             // 200ms
digitalWrite(LED_BUILTIN, HIGH);        // turn the LED on
delay(150);                             // 150ms
digitalWrite(LED_BUILTIN, LOW);         // turn the LED off
delay(500);                             // 500ms
return;
}
// END OF FILE
